@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
+ *  Copyright (C) 2009-2010  Nokia Corporation and/or its subsidiary(-ies).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -287,7 +287,7 @@ static void wran_reachable_cb(const GIsiMessage *msg, void *opaque)
 	if (g_isi_msg_error(msg) < 0)
 		return;
 
-	ISI_VERSION_DBG(msg);
+	ISI_RESOURCE_DBG(msg);
 
 	rd->wran_object = g_isi_msg_object(msg);
 
@@ -304,10 +304,12 @@ static void gss_reachable_cb(const GIsiMessage *msg, void *opaque)
 {
 	struct ofono_radio_settings *rs = opaque;
 
-	if (g_isi_msg_error(msg) < 0)
+	if (g_isi_msg_error(msg) < 0) {
+		ofono_radio_settings_remove(rs);
 		return;
+	}
 
-	ISI_VERSION_DBG(msg);
+	ISI_RESOURCE_DBG(msg);
 
 	ofono_radio_settings_register(rs);
 }
